"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCode = void 0;
const json5_1 = __importDefault(require("json5"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const flat_1 = require("flat");
const prettier_1 = __importDefault(require("prettier"));
const vue_i18n_1 = require("vue-i18n");
function generateCode(source, query, options) {
    const value = merge(parse(source, query), query);
    let code = '';
    const preCompile = !!options.preCompile;
    if (preCompile) {
        code += generateCompiledCode(value);
        code += `export default function (Component) {
  Component.__i18n = Component.__i18n || _getResource
}\n`;
    }
    else {
        code += `export default function (Component) {
  Component.__i18n = Component.__i18n || []
  Component.__i18n.push(${stringify(value)})
}\n`;
    }
    return prettier_1.default.format(code, { parser: 'babel' });
}
exports.generateCode = generateCode;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function stringify(data) {
    return vue_i18n_1.friendlyJSONstringify(data);
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function merge(data, query) {
    if (query.locale && typeof query.locale === 'string') {
        return Object.assign({}, { [query.locale]: data });
    }
    else {
        return data;
    }
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function parse(source, query) {
    const value = Buffer.isBuffer(source) ? source.toString() : source;
    const { lang } = query;
    switch (lang) {
        case 'yaml':
        case 'yml':
            return js_yaml_1.default.safeLoad(value);
        case 'json5':
            return json5_1.default.parse(value);
        default:
            return JSON.parse(value);
    }
}
function generateCompiledCode(messages) {
    let code = '';
    code += `function _register(functions, pathkey, msg) {
  const path = JSON.stringify(pathkey)
  functions[path] = msg
}
`;
    code += `const _getResource = () => {
  const functions = Object.create(null)
`;
    const locales = Object.keys(messages);
    locales.forEach(locale => {
        const message = flat_1.flatten(messages[locale]);
        const keys = Object.keys(message);
        keys.forEach(key => {
            const format = message[key];
            let occured = false;
            const options = {
                mode: 'arrow',
                // TODO: source mapping !
                onError(err) {
                    console.error(err);
                    occured = true;
                }
            };
            const result = vue_i18n_1.baseCompile(format, options);
            if (!occured) {
                code += `  _register(functions, ${vue_i18n_1.generateFormatCacheKey(locale, key, format)}, ${result.code})\n`;
            }
        });
    });
    code += `
  return { functions }
}
`;
    return code;
}
